@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2015-2019 Laurent Montel <montel@kde.org>
+   Copyright (C) 2015-2020 Laurent Montel <montel@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -60,6 +60,19 @@ public:
     RichTextComposer *q = nullptr;
     RichTextComposer::Mode mode = RichTextComposer::Plain;
     bool forcePlainTextMarkup = false;
+    struct UndoHtmlVersion {
+        QString originalHtml;
+        QString plainText;
+        Q_REQUIRED_RESULT bool isValid() const {
+            return !originalHtml.isEmpty() && !plainText.isEmpty();
+        }
+        void clear() {
+            originalHtml.clear();
+            plainText.clear();
+        }
+    };
+    UndoHtmlVersion undoHtmlVersion;
+    bool blockClearUndoHtmlVersion = false;
 };
 
 RichTextComposer::RichTextComposer(QWidget *parent)
@@ -67,6 +80,13 @@ RichTextComposer::RichTextComposer(QWidget *parent)
     , d(new RichTextComposerPrivate(this))
 {
     setAcceptRichText(false);
+    connect(this, &RichTextComposer::textChanged, this, [this]() {
+        if (!d->blockClearUndoHtmlVersion && d->undoHtmlVersion.isValid() && (d->mode == RichTextComposer::Plain)) {
+            if (toPlainText() != d->undoHtmlVersion.plainText) {
+                d->undoHtmlVersion.clear();
+            }
+        }
+    });
 }
 
 RichTextComposer::~RichTextComposer()
@@ -257,6 +277,17 @@ void RichTextComposer::activateRichText()
     if (d->mode == RichTextComposer::Plain) {
         setAcceptRichText(true);
         d->mode = RichTextComposer::Rich;
+        if (d->undoHtmlVersion.isValid() && (toPlainText() == d->undoHtmlVersion.plainText)) {
+            setHtml(d->undoHtmlVersion.originalHtml);
+            d->undoHtmlVersion.clear();
+        } else {
+#if 0 //Need to investigate it
+            //try to import markdown
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+            document()->setMarkdown(toPlainText(), QTextDocument::MarkdownDialectCommonMark);
+#endif
+#endif
+        }
         Q_EMIT textModeChanged(d->mode);
     }
 }
@@ -265,9 +296,13 @@ void RichTextComposer::switchToPlainText()
 {
     if (d->mode == RichTextComposer::Rich) {
         d->mode = RichTextComposer::Plain;
+        d->blockClearUndoHtmlVersion = true;
+        d->undoHtmlVersion.originalHtml = toHtml();
         // TODO: Warn the user about this?
         insertPlainTextImplementation();
         setAcceptRichText(false);
+        d->undoHtmlVersion.plainText = toPlainText();
+        d->blockClearUndoHtmlVersion = false;
         Q_EMIT textModeChanged(d->mode);
     }
 }
