@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019 Montel Laurent <montel@kde.org>
+  Copyright (c) 2019-2020 Laurent Montel <montel@kde.org>
 
   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published by
@@ -34,11 +34,6 @@ QString EmoticonTextEditItem::text() const
     return mText;
 }
 
-QString EmoticonTextEditItem::pixmapPath() const
-{
-    return mPixmapPath;
-}
-
 EmoticonListWidgetSelector::EmoticonListWidgetSelector(QWidget *parent)
     : QListWidget(parent)
 {
